from typing import Dict, Optional

import requests
import json

from e2enetworks.constants import headers
from e2enetworks.constants import BASE_GPU_URL
from e2enetworks.cloud.tir.constants import ARGUMENT_IS_MANDATORY
from e2enetworks.cloud.tir.helpers import plan_name_to_sku_item_price_id
from e2enetworks.cloud.tir.skus import Plans, client
from e2enetworks.constants import PIPELINE
from e2enetworks.cloud.tir.utils import prepare_object


class PipelineClient:
    def __init__(
        self,
        project: Optional[str] = None,
    ):  
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)
        
        if project:
            client.Default.set_project(project)

    def create_pipeline(
        self,
        name,
        file_path,
        description=''
    ):
        if not name or not file_path:
            raise ValueError(f"name and file {ARGUMENT_IS_MANDATORY}")
        file = open(file_path, 'rb')
        files = [
            ('uploadfile',
             (file.name, file, 'application/octet-stream'))
        ]
        url = f"{client.Default.gpu_projects_path()}/pipelines/upload/?name={name}&description={description}&"
        req = requests.Request('POST', url, files=files)
        response = client.Default.make_request(req)
        return prepare_object(response)
    
    def list_pipelines(self):
        url = f"{client.Default.gpu_projects_path()}/pipelines/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get_pipeline(
        self,
        pipeline_id: str = ''
    ):
        if not pipeline_id:
            raise ValueError(f"PIPELINE_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/{pipeline_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete_pipeline(
        self,
        pipeline_id: str = ''
    ):
        if not pipeline_id:
            raise ValueError(f"PIPELINE_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/{pipeline_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    # APIS for Pipeline Versions

    def create_pipeline_version(
        self,
        name,
        file_path,
        pipeline_id: str = '',
        description=''
    ):
        if not name:
            raise ValueError(f"name {ARGUMENT_IS_MANDATORY}")
        if not file_path:
            raise ValueError(f"yaml file {ARGUMENT_IS_MANDATORY}")
        if not pipeline_id:
            raise ValueError(f"pipeline_id {ARGUMENT_IS_MANDATORY}")

        file = open(file_path, 'rb')
        files = [
            ('uploadfile',
             (file.name, file, 'application/octet-stream'))
        ]
        url = f"{client.Default.gpu_projects_path()}/pipelines/upload/?name={name}&description={description}&" \
              f"pipeline_id={pipeline_id}&"
        req = requests.Request('POST', url, files=files)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list_pipeline_version(
            self,
            pipeline_id: str = ''
    ):
        if not pipeline_id:
            raise ValueError(f"PIPELINE_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/{pipeline_id}/versions/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get_pipeline_version(
        self,
        pipeline_id: str = '',
        version_id: str = ''
    ):
        if not pipeline_id:
            raise ValueError(f"PIPELINE_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/{pipeline_id}/versions/{version_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete_pipeline_version(
        self,
        pipeline_id: str = '',
        version_id: str = ''
    ):
        if not pipeline_id:
            raise ValueError(f"PIPELINE_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/{pipeline_id}/versions/{version_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def create_experiment(
        self,
        name: str,
        description: Optional[str] = None,
    ):
        request_params = {
            'name': name,
            'description': description,
        }
        url = f"{client.Default.gpu_projects_path()}/pipelines/experiments/"
        req = requests.Request('POST', url, data=request_params)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list_experiments(
        self
    ):
        url = f"{client.Default.gpu_projects_path()}/pipelines/experiments/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get_experiment(
        self,
        experiment_id: str = '',
    ):
        if not experiment_id:
            raise ValueError(f"EXPERIMENT_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/experiments/{experiment_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete_experiment(
        self,
        experiment_id: str = '',
    ):

        if not experiment_id:
            raise ValueError(f"EXPERIMENT_ID {ARGUMENT_IS_MANDATORY}")

        url = f"{client.Default.gpu_projects_path()}/pipelines/experiments/{experiment_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def create_run(
        self,
        name: str,
        plan_name=None,
        description: str = '',
        experiment_id: str = '',
        pipeline_version_id: str = '',
    ):
        if not plan_name:
            return ValueError(f"plan_name is necessary")
        if not name or not experiment_id or not pipeline_version_id:
            raise ValueError(f"name, experiment_id, pipeline_version_id {ARGUMENT_IS_MANDATORY}")
        skus, skus_table = Plans().get_plans_list(PIPELINE)
        payloads = json.dumps({"name": name,
                    "description": description,
                    "experiment_id": experiment_id,
                    "sku_item_price_id": plan_name_to_sku_item_price_id(skus, plan_name),
                    "pipeline_version_id": pipeline_version_id})
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{client.Default.gpu_projects_path()}/pipelines/runs/?apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payloads)
        return response

    def list_runs(self):
        url = f"{client.Default.gpu_projects_path()}/pipelines/runs/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get_run(self, run_id):
        if not run_id:
            raise ValueError(f"RUN_ID {ARGUMENT_IS_MANDATORY}")
        url = f"{client.Default.gpu_projects_path()}/pipelines/runs/{run_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete_run(self, run_id):
        if not run_id:
            raise ValueError(f"RUN_ID {ARGUMENT_IS_MANDATORY}")

        url = f"{client.Default.gpu_projects_path()}/pipelines/runs/{run_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        help_text = """
        PipelineClient Class Help:

        This class provides methods for interacting with pipeline-related operations.
        Before using these methods, make sure to initialize the client using:
        - Using e2enetworks.cloud.tir.init(...)

        Available Methods:
        1. create_pipeline(name, description, file_path)
           - Create a new pipeline.
        2. list_pipelines()
           - List existing pipelines.
        3. get_pipeline(pipeline_id)
           - Get details of a specific pipeline.
        4. delete_pipeline(pipeline_id)
           - Delete a specific pipeline.
        5. create_pipeline_version(name, description, file_path, pipeline_id)
           - Create a new pipeline version.
        6. list_pipeline_version(pipeline_id)
           - List all versions of a pipeline.
        7. get_pipeline_version(pipeline_id, version_id)
           - Get details of a specific pipeline version.
        8. delete_pipeline_version(pipeline_id, version_id)
           - Delete a specific pipeline version.
        9. create_experiment(name, description=None)
           - Create a new experiment.
        10. list_experiments()
           - List existing experiments.
        11. get_experiment(experiment_id='')
           - Get details of a specific experiment.
        12. delete_experiment(experiment_id='')
           - Delete a specific experiment.
        13. create_run(name='', description='', experiment_id='', pipeline_version_id='', service_account='')
           - Create a new run.
        14. list_runs()
            - List existing runs.
        15. get_run(run_id='')
            - Get details of a specific run.
        16. delete_run(run_id='')
            - Delete a specific run.

        Note: Certain methods require specific arguments. Refer to the method signatures for details.
        """
        print(help_text)



import requests

from e2enetworks.constants import BASE_GPU_URL
from typing import Optional
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.utils import prepare_object


class APITokens:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def create(self, token_name):

        if not token_name:
            raise ValueError(token_name)

        payload = {
            "token_name": token_name,
        }
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/auth-token/?project_id=" \
              f"{client.Default.project()}&"

        req = requests.Request('POST', url, data=payload)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/auth-token/?project_id={client.Default.project()}&"

        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, token_id):
        if not token_id:
            raise ValueError(token_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/auth-token/{token_id}/?project_id=" \
              f"{client.Default.project()}&"

        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("APITokens Class Help")
        print("\t\t====================")
        print("\t\tThis class provides functionalities to manage API tokens.")
        print("\t\tAvailable methods:")
        print(
            "\t\t1. __init__(team, project): Initializes an APITokens instance with the specified team ID and "
            "project ID.")
        print("\t\t2. create(token_name): Creates a new API token with the provided authentication token.")
        print("\t\t3. list(): Lists all API tokens associated with the team and project.")
        print("\t\t4. delete(token_id): Deletes an API token with the given token ID.")
        print("\t\t7. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tapi_tokens = APITokens(123, 456)")
        print("\t\tapi_tokens.create('token_name')")
        print("\t\tapi_tokens.list()")
        print("\t\tapi_tokens.delete(789)")



import requests
from prettytable import PrettyTable

from e2enetworks.cloud.tir import client
from e2enetworks.constants import BASE_GPU_URL


class Images:
    def __init__(self):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

    def list(self, is_jupyterlab_enabled: bool):
        url = f"{BASE_GPU_URL}gpu_service/image/?is_jupyterlab_enabled={is_jupyterlab_enabled}&"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        if response.status_code == 200:
            images = response.json()["data"]
            images_table = PrettyTable()
            images_table.field_names = ['Name', 'version', 'image_id','image_version_id']
            for image in images:
                for v in image["versions"]:
                    images_table.add_row([image.get('name'), v["version"],  image.get('image_id'), v['image_version_id']])
            return images_table


    @staticmethod
    def help():
        print("Images Class Help")
        print("\t\t================")
        print("\t\tThis class provides functionalities to interact with Images.")
        print("\t\tAvailable methods:")

        print("\t\t1. list(): Lists all Images having JupyterLab enabled/disabled.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\timages = Images()")
        print("\t\timages.list(is_jupyterlab_enabled: bool(True/False))")



from typing import Dict, Optional

import requests
from prettytable import PrettyTable

from e2enetworks.cloud.tir.finetuning.constants import HUGGING_FACE, EOS_BUCKET, TEXT_MODELS_LIST
from e2enetworks.cloud.tir.helpers import get_argument_from_kwargs
from e2enetworks.cloud.tir.skus import Plans, client
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.constants import PIPELINE


class FinetuningClient():
    def __init__(self,
                 project: Optional[str] = None,
                 ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

    def create_finetuning(self,
                          name,
                          model_name,
                          plan_name,
                          huggingface_integration_id,
                          wandb_integration_id=None,
                          wandb_integration_run_name="",
                          description=None,
                          training_type="Peft",
                          **kwargs
                          ):
        if not isinstance(plan_name, str):
            return ValueError("plan_name is should be string")

        if model_name in TEXT_MODELS_LIST:
            training_inputs = self._get_text_model_inputs(**kwargs)
        else:
            raise Exception(f'model_name is  invalid : {model_name}')

        payload = {"name": name,
                    "model_name": model_name,
                    "huggingface_integration_id": huggingface_integration_id,
                    "sku_item_price_id": self._get_sku_item_price_from_plan_name(plan_name),
                    "training_inputs": training_inputs,
                    "training_type": training_type,
                    "wandb_integration_id": wandb_integration_id if wandb_integration_id else None,
                    "wandb_integration_run_name": wandb_integration_run_name,
                    "description": description
                    }
        url = f"{client.Default.gpu_projects_path()}/finetuning/?"
        req = requests.Request('POST', url, json=payload)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def _get_text_model_inputs(self, **kwargs):
        return {**self._get_dataset_config(**kwargs),
                "validation_split_ratio": get_argument_from_kwargs('validation_split_ratio', kwargs, float, 0.1),
                "target_dataset_field": get_argument_from_kwargs("target_dataset_field", kwargs, str, 'text'),
                "gradient_accumulation_steps": get_argument_from_kwargs("gradient_accumulation_steps", kwargs, int, 1),
                "context_length": get_argument_from_kwargs('context_length', kwargs, int, 512),
                "learning_rate": get_argument_from_kwargs('learning_rate', kwargs, float, 0.0000141),
                "epochs": get_argument_from_kwargs('epochs', kwargs, int, 1),
                "stop_training_when": get_argument_from_kwargs('stop_training_when', kwargs, str, "epoch_count"),
                "max_steps": get_argument_from_kwargs('max_steps', kwargs, int, -1),
                "batch_size": get_argument_from_kwargs('batch_size', kwargs, int, 4),
                "peft_lora_alpha": get_argument_from_kwargs('peft_lora_alpha', kwargs, int, 16),
                "peft_lora_r": get_argument_from_kwargs('peft_lora_r', kwargs, int, 64),
                "max_grad_norm": get_argument_from_kwargs('max_grad_norm', kwargs, int, 1),
                "save_strategy": get_argument_from_kwargs('save_strategy', kwargs, str, "no"),
                "task": get_argument_from_kwargs('task', kwargs, str, "Instruction-Finetuning"),
                "prompt_configuration": get_argument_from_kwargs('prompt_configuration', kwargs, str, ""),
                "save_steps": get_argument_from_kwargs('save_steps', kwargs, int, 10),
                "limit_training_records_count": get_argument_from_kwargs('limit_training_records_count', kwargs, int, 10),
                "limit_eval_records_count": get_argument_from_kwargs('limit_training_records_count', kwargs, int, 10),
                }

    def _get_dataset_config(self, **kwargs):
        dataset_type = kwargs.get("dataset_type")
        dataset_info = kwargs.get("dataset")

        if dataset_type not in [EOS_BUCKET, HUGGING_FACE]:
            raise Exception(f"dataset_type: only {EOS_BUCKET}, {HUGGING_FACE} allowed")

        if dataset_type == HUGGING_FACE:
            return {"dataset_type": HUGGING_FACE,
                    "dataset": dataset_info}

        dataset_sub_str = dataset_info.split('/')
        if len(dataset_sub_str) <= 1:
            raise Exception("dataset invalid")
        object_name = dataset_info.replace(f"{dataset_sub_str[0]}/", '', 1)
        if not object_name:
            raise Exception("dataset invalid")
        return {"dataset_type": dataset_type,
                "dataset": f'{dataset_sub_str[0]}/{object_name}'}

    def list_finetunings(self):
        url = f"{client.Default.gpu_projects_path()}/finetuning/?"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list_supported_models(self):
        url = f"{client.Default.gpu_projects_path()}/finetuning/model_types/?"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def show_plan_names(self):
        plans = Plans()
        plans_list = plans.list(PIPELINE, image='')
        gpu_skus = plans_list["GPU"]
        plans_table = PrettyTable()
        plans_table.field_names = ['name', 'series', 'cpu', 'gpu', 'memory',
                                   'sku_item_price_id', 'sku_type', 'committed_days', 'unit_price']
        plans.insert_plans_in_table(gpu_skus, plans_table)
        print(plans_table)

    def _get_sku_item_price_from_plan_name(self, plan_name, committed_days=0):
        plans = Plans().list(PIPELINE, image='')
        plan_name_to_sku = {}
        gpu_skus = plans["GPU"]
        for sku in gpu_skus:
            for sku_item_price in sku["plans"]:
                if not sku["is_free"]:
                    name = sku.get('name')
                    committed_days = sku_item_price.get('committed_days')
                    key = f'{name}_c{committed_days}'
                    plan_name_to_sku[key] = sku_item_price['sku_item_price_id']
        if plan_name_to_sku.get(f'{plan_name}_c{committed_days}'):
            return plan_name_to_sku.get(f'{plan_name}_c{committed_days}')
        raise Exception(f'Plan_name invalid : {plan_name}')

    def get_finetuning(self,
                       finetuning_id: str | int
                       ):
        url = f"{client.Default.gpu_projects_path()}/finetuning/{finetuning_id}/?"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete_finetuning(self,
                          finetuning_id: str | int
                          ):
        url = f"{client.Default.gpu_projects_path()}/finetuning/{finetuning_id}/?"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def stop_finetuning(self,
                        finetuning_id: str | int
                        ):
        url = f"{client.Default.gpu_projects_path()}/finetuning/{finetuning_id}/?&action=terminate&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def retry_finetuning(self,
                         finetuning_id: str | int
                         ):
        url = f"{client.Default.gpu_projects_path()}/finetuning/{finetuning_id}/?&action=retry&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        help_text = """
        FinetuningClient Class Help:

        This class provides methods for interacting with finetuning-related operations.
        Before using these methods, make sure to initialize the client using:
        - Using e2enetworks.cloud.tir.init(...)

        Available Methods:
        1. create_finetuningcreate_finetuning(self,
                          name,
                          model_name,
                          plan_name,
                          huggingface_integration_id,
                          wandb_integration_id=None,
                          wandb_integration_run_name="",
                          description="",
                          training_type="Peft",
                          **kwargs
                          )
           - Create a new finetuning.
        2. list_finetunings()
           - List existing finetunings.
        3. get_finetuning(finetuning_id)
           - Get details of a specific finetuning.
        4. delete_finetuning(finetuning_id)
           - Delete a specific finetuning.
        5. stop_finetuning(finetuning_id)
           - stop a specific finetuning.
        6. retry_finetuning(finetuning_id)
           - retry a failed/terminated finetuning.
        7. show_plan_names()
           - list currently supported skus for finetuning
        8. list_supported_models()
           - list currently supported models for finetuning
        Note: Certain methods require specific arguments. Refer to the method signatures for details.
        """
        print(help_text)



import json
from typing import Optional

import requests
import os

from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.minio_service import MinioService
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.constants import BASE_GPU_URL, BUCKET_TYPES, headers


class Datasets:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = "",
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def create(self, name=None, bucket_name=None, bucket_type="managed", description=""):
        if bucket_type!="managed":
            if not bucket_name:
                raise ValueError("please specify a valid bucket_name")
            
            if not bucket_type:
                raise ValueError("please specify a valid bucket_type")

        payload = json.dumps({
            "type": bucket_type,
            "name": name,
            "bucket_name": bucket_name,
            "description": description,
        })
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/datasets/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        return prepare_object(response)

    def get(self, dataset_id):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/datasets/" \
              f"{dataset_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/datasets/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, dataset_id):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/datasets/" \
              f"{dataset_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)
    
    def upload_dataset(self, dataset_path, prefix="", dataset_id=None):
        dataset = self.get(dataset_id) if dataset_id else None
        if not dataset:
            raise ValueError("invalid dataset_id parameter")
        access_key = dataset.access_key.access_key
        secret_key = dataset.access_key.secret_key
        try:
            minio_service = MinioService(access_key=access_key, secret_key=secret_key)
            if os.path.isdir(dataset_path):
                minio_service.upload_directory_recursive(bucket_name=dataset.bucket.bucket_name,
                                                    source_directory=dataset_path, prefix=prefix)
            else:
                minio_service.upload_file(bucket_name=dataset.bucket.bucket_name, file_path=dataset_path, prefix=prefix)
            print("Dataset Pushed Successfully")
        except Exception as e:
            print(e)

        return dataset

    def download_dataset(self, dataset_id, local_path, prefix=""):
        dataset = self.get(dataset_id) if dataset_id else None
        if not dataset:
                raise ValueError("Invalid dataset id")
        access_key = dataset.access_key.access_key
        secret_key = dataset.access_key.secret_key
        try:
            minio_service = MinioService(access_key=access_key, secret_key=secret_key)
            minio_service.download_directory_recursive(bucket_name=dataset.bucket.bucket_name,
                                                       local_path=local_path, prefix=prefix)
            print("dataset downloaded successfully")
        except Exception as e:
            print(e)

        return dataset

    @staticmethod
    def help():
        print("Datasets Class Help")
        print("\t\t=================")
        print("\t\tThis class provides functionalities to interact with Datasets.")
        print("\t\tAvailable methods:")
        print(
            "\t\t1. __init__(team, project): Initializes a Datasets instance with the specified team and "
            "project IDs.")
        print(f"\t\t2. create(name, bucket_name=, bucket_type, description): Creates a new dataset with the provided"
              f"name, bucket name, bucket type and description\n Bucket Name is not required in case of"
              f" bucket_type='managed'")
        print("\t\t3. get(bucket_name): Retrieves information about a specific dataset using its bucket name.")
        print("\t\t4. list(): Lists all datasets associated with the team and project.")
        print("\t\t5. delete(dataset_id): Deletes a dataset with the given dataset_id")
        print("\t\t3. push_dataset(dataset_path, prefix, dataset_id): Creates a new dataset with the provided "
              "details.")
        print("\t\t4. download_dataset(dataset_id, local_path, prefix)")
        print("\t\t8. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tdatasets = Datasets(123, 456)")
        print(f"\t\tdatasets.create(name='Test Dataset', bucket_name='dataset-bucket', bucket_type={BUCKET_TYPES},"
              f" description='Test Dataset')")
        print("\t\tdatasets.get('Bucket Name')")
        print("\t\tdatasets.list()")
        print("\t\tdatasets.delete(236)")
        print(f"\t\tdatasets.push_dataset(dataset_path, prefix='', dataset_id=None)")
        print(f"\t\tdatasets.download_dataset(dataset_id=<dataset id>, local_path=<path of local directory>,"
              f" prefix=<prefix in the bucket>)")


HUGGING_FACE = "huggingface"
EOS_BUCKET = "eos-bucket"


LLAMA2 = "meta-llama/Llama-2-7b-hf"
MISTRAL_7B = "mistralai/Mistral-7B-v0.1"
STABLE_DIFFUSION_2_1 = "stabilityai/stable-diffusion-2-1"
MISTRAL_8X7B = "mistralai/Mixtral-8x7B-v0.1"
GEMMA_7B = "google/gemma-7b"
LLAMA3_8B = "meta-llama/Meta-Llama-3-8B"
TEXT_MODELS_LIST = [LLAMA2, LLAMA3_8B, MISTRAL_7B, MISTRAL_8X7B, GEMMA_7B]



import os

BASE_URL_MODEL_API_CLIENT = os.environ.get("MODEL_API_CLIENT_HOST", "https://infer.e2enetworks.net/")
MY_ACCOUNT_LB_URL = os.environ.get("E2E_TIR_API_HOST", "https://api.e2enetworks.com/myaccount/")

GPU_URL = "api/v1/gpu/"
BASE_GPU_URL = f"{MY_ACCOUNT_LB_URL}{GPU_URL}"
VALIDATED_SUCCESSFULLY = "Validated Successfully"
INVALID_CREDENTIALS = "Validation Failed, Invalid APIkey or Token"
headers = {
    'Connection': 'keep-alive',
    'Content-Type': 'application/json',
    'Origin': 'https://thor-gpu.e2enetworks.net',
    'Referer': 'https://thor-gpu.e2enetworks.net/',
    'Sec-Fetch-Dest': 'empty',
    'Sec-Fetch-Mode': 'cors',
    'Sec-Fetch-Site': 'same-site',
}
MANAGED_STORAGE = "managed"
E2E_OBJECT_STORAGE = "e2e_s3"
BUCKET_TYPES = [MANAGED_STORAGE, E2E_OBJECT_STORAGE]
BUCKET_TYPES_HELP = {
    MANAGED_STORAGE: "To Create New Bucket",
    E2E_OBJECT_STORAGE: " To Use Existing Bucket"
}
NOTEBOOK = "notebook"
INFERENCE = "inference_service"
PIPELINE = "pipeline"
VECTOR_DB = "vector_db"

PRIVATE = "private"
CUSTOM = "custom"
REGISTRY = "registry.e2enetworks.net"
FREE_USAGE = "free_usage"
PAID_USAGE = "paid_usage"
INSTANCE_TYPE = [FREE_USAGE, PAID_USAGE]
TRITON = "triton"
TENSORRT = "tensorrt"
PYTORCH = "pytorch"
MODEL_TYPES = ['pytorch', 'triton', 'custom']
S3_ENDPOINT = "objectstore.e2enetworks.net"

WHISPER_DATA_LIMIT_BYTES = 50000000
WHISPER_LARGE_V3 = "whisper-large-v3"
LLAMA_2_13B_CHAT = "llama-2-13b-chat"
STABLE_DIFFUSION_2_1 = "stable-diffusion-2-1"
MIXTRAL_8X7B_INSTRUCT = "mixtral-8x7b-instruct"
CODELLAMA_13B_INSTRUCT = "codellama-13b-instruct"
E5_MISTRAL_7B_INSTRUCT = "e5-mistral-7b-instruct"
AUTO_RENEW_STATUS = 'auto_renew'
AUTO_TERMINATE_STATUS = 'auto_terminate'
CONVERT_TO_HOURLY_BILLING = 'convert_to_hourly_billing'
LLAMA_3_8B_INSTRUCT = "llama-3-8b-instruct"
LLMA = "llma"
STABLE_DIFFUSION = "stable_diffusion"
MPT = "mpt"
CODE_LLAMA = "codellama"
FINETUNED = "finetuned"
MIXTRAL8X7B = 'mixtral8x7b'
MIXTRAL7B = 'mixtral7b'
MIXTRAL7B_INSTRUCT = "mistral-7b-instruct"
MIXTRAL8X7B_INSTRUCT = "mixtral-8x7b-instruct"
GEMMA_7B_IT = "gemma-7b-it"
GEMMA_7B = "gemma-7b"
GEMMA_2B = "gemma-2b"
GEMMA_2B_IT = "gemma-2b-it"
PHI_3_MINI_128K_INSTRUCT = "Phi-3-mini-128k-instruct"
STARCODER2_7B = "starcoder2-7b"
VLLM = "vllm"
NEMO_RAG_SERVICE = 'nemo-rag'
STABLE_VIDEO_DIFFUSION_XT = "stable-video-diffusion-img2vid-xt"
STABLE_DIFFUSION_XL = "stable_diffusion_xl"
YOLO = 'yolov8'
CUSTOM = 'custom'
TIR_CUSTOM_FRAMEWORKS = [LLMA, STABLE_DIFFUSION, STABLE_DIFFUSION_XL, YOLO, MPT, CODE_LLAMA, MIXTRAL7B, MIXTRAL8X7B, MIXTRAL7B_INSTRUCT, MIXTRAL8X7B_INSTRUCT, GEMMA_7B_IT, GEMMA_7B, GEMMA_2B, GEMMA_2B_IT, TENSORRT, PYTORCH, TRITON, LLAMA_3_8B_INSTRUCT, VLLM, STARCODER2_7B, PHI_3_MINI_128K_INSTRUCT, NEMO_RAG_SERVICE, STABLE_VIDEO_DIFFUSION_XT, CUSTOM]  # custom model inferences provided by E2E
ALLOWED_CONTAINER_IMAGES = {
    STARCODER2_7B: 'vllm/vllm-openai:latest',
    GEMMA_7B_IT: 'vllm/vllm-openai:latest',
    GEMMA_2B_IT: 'vllm/vllm-openai:latest',
    LLMA: 'vllm/vllm-openai:latest',
    STABLE_DIFFUSION: 'registry.e2enetworks.net/aimle2e/stable-diffusion-2-1:hf-v1',
    MPT: 'vllm/vllm-openai:latest',
    CODE_LLAMA: 'vllm/vllm-openai:latest',
    STABLE_DIFFUSION_XL: 'registry.e2enetworks.net/aimle2e/stable-diffusion-xl-base-1.0:hf-v1',
    MIXTRAL7B_INSTRUCT: 'vllm/vllm-openai:latest',
    MIXTRAL8X7B_INSTRUCT: 'vllm/vllm-openai:latest',
    TENSORRT: ['aimle2e/tritonserver:24.02-trtllm-python-py3-01', 'aimle2e/tritonserver:24.01-trtllm-python-py3-01', 'aimle2e/tritonserver:23.12-trtllm-python-py3-01', 'aimle2e/tritonserver:23.11-trtllm-python-py3-01', 'aimle2e/tritonserver:23.10-trtllm-python-py3-01',],
    PYTORCH: ['pytorch/torchserve:0.9.0', 'pytorch/torchserve:0.8.1', 'pytorch/torchserve:0.8.2',],
    TRITON: ['aimle2e/tritonserver:24.02-py3-01', 'aimle2e/tritonserver:24.01-py3-01', 'aimle2e/tritonserver:23.12-py3-01', 'aimle2e/tritonserver:23.01-py3-01', 'aimle2e/tritonserver:23.10-py3-01',],
    LLAMA_3_8B_INSTRUCT: 'vllm/vllm-openai:latest',
    VLLM: 'vllm/vllm-openai:latest',
    PHI_3_MINI_128K_INSTRUCT: 'vllm/vllm-openai:latest',
    STABLE_VIDEO_DIFFUSION_XT: 'aimle2e/stable-video-diffusion:v1',
    YOLO: 'registry.e2enetworks.net/aimle2e/yolov8:v1',
}

SSH_ENABLE = "enable"
SSH_DISABLE = "disable"
SSH_UPDATE = "update"



import requests

from e2enetworks.constants import BASE_GPU_URL, VECTOR_DB
from e2enetworks.cloud.tir.helpers import plan_name_to_sku_item_price_id
from e2enetworks.cloud.tir.skus import client, Plans
from e2enetworks.cloud.tir.utils import prepare_object


class VectorDB:

    def __init__(self, team=None, project=None):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)
        if project:
            client.Default.set_project(project)
        if team:
            client.Default.set_team(team)

    def create(
        self,
        name,
        plan_name,
        replicas=3,
    ):
        skus_list, _ = Plans().get_plans_list(VECTOR_DB)
        sku_item_price_id = plan_name_to_sku_item_price_id(skus_list, plan_name)
        if not sku_item_price_id:
            raise ValueError("Enter a valid plan_name.")

        req = requests.Request(
            method='POST',
            url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/",
            json={
                "disk_size": 30,
                "name": name,
                "replicas": replicas,
                "sku_item_price_id": sku_item_price_id
            }
        )
        return prepare_object(client.Default.make_request(req))

    def list(self):
        response = client.Default.make_request(
            requests.Request(
                method='GET',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/"
            )
        )
        return prepare_object(response)

    def details(self, vectordb_id):
        response = client.Default.make_request(
            requests.Request(
                method='GET',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/"
            )
        )
        if response.status_code == 200:
            return response.json()["data"]
        return prepare_object(response)

    def delete(self, vectordb_id):
        response = client.Default.make_request(
            requests.Request(
                method='DELETE',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/"
            )
        )
        return prepare_object(response)

    def upgrade_nodes(self, vectordb_id, replicas):
        response = client.Default.make_request(
            requests.Request(
                method='PUT',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/",
                json={"replicas": replicas}
            )
        )
        return prepare_object(response)

    def upgrade_disk_size(self, vectordb_id, disk_size):
        response = client.Default.make_request(
            requests.Request(
                method='PUT',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/pvcs/",
                json={"disk_size": disk_size}
            )
        )
        return prepare_object(response)

    def create_snapshot(self, vectordb_id):
        response = client.Default.make_request(
            requests.Request(
                method='POST',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/snapshot/",
                json={}
            )
        )
        return prepare_object(response)

    def list_snapshots(self, vectordb_id):
        response = client.Default.make_request(
            requests.Request(
                method='GET',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/snapshot/",
            )
        )
        return prepare_object(response)

    def restore_snapshot(self, vectordb_id, snapshot_id):
        response = client.Default.make_request(
            requests.Request(
                method='PUT',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/snapshot/{snapshot_id}/restore/",
            )
        )
        return prepare_object(response)

    def delete_snapshot(self, vectordb_id, snapshot_id):
        response = client.Default.make_request(
            requests.Request(
                method='DELETE',
                url=f"{BASE_GPU_URL}/teams/{client.Default.team()}/projects/{client.Default.project()}/vector_db/{vectordb_id}/snapshot/{snapshot_id}/",
            )
        )
        return prepare_object(response)

    @staticmethod
    def help():
        print("\t\tVectorDB Class Help")
        print("\t\t===================")
        help_text = """
                A class to manage operations on Vector Databases (VectorDB).

                This class provides methods to create, list, and manage vector databases,
                including details about the database, upgrading resources, and snapshot management.

                Methods:
                --------
                1. create(name: str, plan_name: str, replicas(optional): int)
                    Creates a new vector database with the specified name, size, and replicas.

                2. list()
                    Lists all existing vector databases.

                3. details(vectordb_id: int)
                    Retrieves details of a specific vector database given its ID.

                4. delete(vectordb_id: int)
                    Deletes a vector database specified by its ID.

                5. upgrade_nodes(vectordb_id: int, replicas: int)
                    Upgrades the number of replicas for the specified vector database.

                6. upgrade_disk_size(vectordb_id: int, disk_size: int)
                    Upgrades the disk size for the specified vector database.

                7. create_snapshot(vectordb_id: int)
                    Creates a snapshot for the specified vector database.

                8. list_snapshots(vectordb_id: int)
                    Lists all snapshots for the specified vector database.

                9. restore_snapshot(vectordb_id: int, snapshot_id: int)
                    Restores a vector database to a specific snapshot state.

                10. delete_snapshot(vectordb_id: int, snapshot_id: int)
                    Deletes a specific snapshot for the vector database.

                Examples:
                ---------

                # Create a new VectorDB
                vdb = VectorDB()
                vdb.create("my_db", "CPU-C3-4-8GB-0", 5)

                # List all VectorDBs
                vdb.list()

                # Get details of a specific VectorDB
                details = vdb.details(vectordb_id)

                # Delete a VectorDB
                vdb.delete(vectordb_id)

                # Upgrade the number of replicas
                vdb.upgrade_nodes(vectordb_id, 5)

                # Upgrade the disk size
                vdb.upgrade_disk_size(vectordb_id, 200)

                # Create a snapshot
                vdb.create_snapshot(vectordb_id)

                # List all snapshots
                vdb.list_snapshts(vectordb_id)

                # Restore a snapshot
                vdb.restore_snapshot(vectordb_id, snapshot_id)

                # Delete a snapshot
                vdb.delete_snapshot(vectordb_id, snapshot_id)
                """
        print(help_text)



import json
from typing import Optional

import requests

from e2enetworks.cloud.tir.skus import Plans, client
from e2enetworks.cloud.tir.helpers import plan_to_sku_id, convert_to_base64
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.constants import (BASE_GPU_URL, INFERENCE, PYTORCH, TRITON,TENSORRT,PRIVATE,REGISTRY,
                                   headers, TIR_CUSTOM_FRAMEWORKS, ALLOWED_CONTAINER_IMAGES, STABLE_DIFFUSION,
                                   STABLE_DIFFUSION_XL, STABLE_VIDEO_DIFFUSION_XT)

containers = {
    "stable-video-diffusion-img2vid-xt_eos": "aimle2e/stable-video-diffusion:v1_eos",
    "stable_diffusion_eos": "registry.e2enetworks.net/aimle2e/stable-diffusion-2-1:eos-v1",
    "stable_diffusion_xl_eos": "registry.e2enetworks.net/aimle2e/stable-diffusion-xl-base-1.0:eos-v1",
}
eos_framework = [STABLE_VIDEO_DIFFUSION_XT, STABLE_DIFFUSION, STABLE_DIFFUSION_XL]


class EndPoints:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def list_plans(self,framework):
        if framework not in TIR_CUSTOM_FRAMEWORKS:
            raise ValueError(f"framework {framework} is not supported. framework should be one of: %s" % TIR_CUSTOM_FRAMEWORKS)
        return Plans().list_endpoint_plans(framework=framework)
    
    def list_frameworks(self):
        print(TIR_CUSTOM_FRAMEWORKS)
        return TIR_CUSTOM_FRAMEWORKS

    def get_container_name(self, framework, container_name=None, model_id=None):
        if framework not in TIR_CUSTOM_FRAMEWORKS:
            raise ValueError(f"framework {framework} is not supported. framework should be one of: %s" % TIR_CUSTOM_FRAMEWORKS)
        if framework == "custom":
            return container_name
        if model_id and framework in eos_framework:
            return containers[framework+'_eos'] if framework+'_eos' in containers else None
        else:
            return ALLOWED_CONTAINER_IMAGES[framework] if framework in TIR_CUSTOM_FRAMEWORKS else None
        
    def get_container_type(self, framework):
        if framework in TIR_CUSTOM_FRAMEWORKS:
            return 'public'
        else:
            return 'private'

    def create_triton(self, endpoint_name, plan, model_id, container_name=None,
                                        model_path='', is_auto_scale_enabled=False, metric="cpu",
                                        replicas=1, commands=[], value=12, disk_size=30, env_variables=[],
                                        world_size=1, mount_path='', min_replicas=1, max_replicas=2):
        if not endpoint_name:
            raise ValueError(endpoint_name)
        if not plan:
            raise ValueError(plan)
        if not model_id:
            raise ValueError(model_id)

        args = "['mpirun','--allow-run-as-root','-n', '1','tritonserver','--exit-on-error=false','--model-store=/mnt/models','--grpc-port=9000','--http-port=8080','--metrics-port=8082','--allow-grpc=true','--allow-http=true']"
        args = convert_to_base64(args)
        commands = convert_to_base64(str(commands))
        return self.create_inference_for_framework(endpoint_name=endpoint_name,
                                                   plan=plan,
                                                   model_path=model_path,
                                                   model_id=model_id,
                                                   commands=commands,
                                                   value=value,
                                                   disk_size=disk_size,
                                                   env_variables=env_variables,
                                                   world_size=world_size,
                                                   mount_path=mount_path,
                                                   min_replicas=min_replicas,
                                                   max_replicas=max_replicas,
                                                   replicas=replicas,
                                                   framework=TRITON,
                                                   container_name=container_name,
                                                   metric=metric,
                                                   args=args,
                                                   is_auto_scale_enabled=is_auto_scale_enabled)

    def create_tensorrt(self, endpoint_name, plan, model_id, container_name,
                                        model_path='', is_auto_scale_enabled=False, metric="cpu",
                                        replicas=1, commands=[], value=12, disk_size=30, env_variables=[],
                                        world_size=1, mount_path='', min_replicas=1, max_replicas=2):
        if not endpoint_name:
            raise ValueError(endpoint_name)
        if not plan:
            raise ValueError(plan)
        if not model_id:
            raise ValueError(model_id)

        args = "['mpirun','--allow-run-as-root','-n', '1','tritonserver','--exit-on-error=false','--model-store=/mnt/models','--grpc-port=9000','--http-port=8080','--metrics-port=8082','--allow-grpc=true','--allow-http=true']"
        args = convert_to_base64(args)
        commands = convert_to_base64(str(commands))
        return self.create_inference_for_framework(endpoint_name=endpoint_name,
                                                   plan=plan,
                                                   model_path=model_path,
                                                   model_id=model_id,
                                                   commands=commands,
                                                   value=value,
                                                   disk_size=disk_size,
                                                   env_variables=env_variables,
                                                   world_size=world_size,
                                                   mount_path=mount_path,
                                                   min_replicas=min_replicas,
                                                   max_replicas=max_replicas,
                                                   replicas=replicas,
                                                   framework=TENSORRT,
                                                   container_name=container_name,
                                                   metric=metric,
                                                   args=args,
                                                   is_auto_scale_enabled=is_auto_scale_enabled)

    def create_pytorch(self, endpoint_name, plan, model_id, container_name,
                                        model_path='', is_auto_scale_enabled=False,
                                        metric="cpu", replicas=1, commands=[], value=12, disk_size=30, env_variables=[], 
                                        world_size=1, mount_path='', min_replicas=1, max_replicas=2):
        if not endpoint_name:
            raise ValueError(endpoint_name)
        if not plan:
            raise ValueError(plan)
        if not model_id:
            raise ValueError(model_id)
        
        args = "['torchserve','--start', '--model-store=/mnt/models/model-store','--ts-config=/mnt/models/config/config.properties']"
        args = convert_to_base64(args)
        commands = convert_to_base64(str(commands))
        return self.create_inference_for_framework(endpoint_name=endpoint_name,
                                                   container_name=container_name,
                                                   metric=metric,
                                                   args=args,
                                                   commands=commands,
                                                   value=value,
                                                   disk_size=disk_size,
                                                   env_variables=env_variables,
                                                   world_size=world_size,
                                                   mount_path=mount_path,
                                                   min_replicas=min_replicas,
                                                   max_replicas=max_replicas,
                                                   plan=plan,
                                                   model_path=model_path,
                                                   model_id=model_id,
                                                   replicas=replicas,
                                                   framework=PYTORCH,
                                                   is_auto_scale_enabled=is_auto_scale_enabled)

    def create_inference_for_framework(self, endpoint_name, container_name, plan,
                                        model_path, model_id, framework, is_auto_scale_enabled,
                                        args, metric="cpu", replicas=1, commands=[], value=12, disk_size=50, env_variables=[],
                                        world_size=1, mount_path='', min_replicas=1, max_replicas=2):
        skus = Plans().get_skus_list(INFERENCE, framework=framework)
        sku_id = plan_to_sku_id(skus=skus, plan=plan)

        if not sku_id:
            raise ValueError(plan)

        payload = {
            "name": endpoint_name,
            "world_size": world_size,
            "custom_endpoint_details": {
                "container": {
                    "container_name": container_name,
                    "container_type": "public",
                    "advance_config": {
                        "image_pull_policy": "Always",
                    }
                },
                "resource_details": {
                    "disk_size": disk_size,
                    "mount_path": mount_path,
                    "env_variables": env_variables
                },
            },
            "model_id": model_id,
            "sku_id": sku_id,
            "replica": replicas,
            "path": model_path,
            "framework": framework,
            "is_auto_scale_enabled": is_auto_scale_enabled,
            "detailed_info": {
                "commands": commands,
                "args": args,
            },
        }

        if is_auto_scale_enabled:
            rules = [
                {
                    "metric": metric,
                    "condition_type": "limit",
                    "value": value,
                    "watch_period": "60"
                }
            ]
            
            auto_scale_policy = {
                "min_replicas": min_replicas,
                "max_replicas": max_replicas,
                "rules": rules,
                "stability_period": "300"
            }
            payload["auto_scale_policy"] = auto_scale_policy

        payload = json.dumps(payload)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/"
        req = requests.Request('POST', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        if response.ok:
            print(f"To check the Inference Status and logs, PLease visit "
                f"https://gpu-notebooks.e2enetworks.com/projects/{client.Default.project()}/model-endpoints")
        return prepare_object(response)
    
    def registry_namespace_list(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/container_registry/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)
    
    def registry_detail(self, registry_namespace_id):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/container_registry/{registry_namespace_id}/namespace-repository/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def create(self, endpoint_name, framework, plan, is_auto_scale_enabled=False, 
               registry_namespace_id=None, model_id=None, container_name=None, container_type="public",commands=[],args=[],
               replicas=1, disc_size=50, world_size=1, model_path="", env_variables=[], mount_path="", metric="cpu", value=12, min_replicas=1, max_replicas=2,
               hugging_face_id=None, hf_token_ingration_id=None, dataset_id=None, dataset_path=None, initial_delay_seconds=10, period_seconds=10, timeout_seconds=10,
               failure_threshold=3, success_threshold=1, liveness_probe_protocol='http', readiness_probe_protocol='http', probe_url="/metrics", liveness_commands='', readiness_commands=''):
        if not endpoint_name:
            raise ValueError(endpoint_name)
        if not framework:
            raise ValueError(framework)
        if not plan:
            raise ValueError(plan)

        skus = Plans().get_skus_list(INFERENCE, framework=framework)
        sku_id = plan_to_sku_id(skus=skus, plan=plan)
        if not sku_id:
            raise ValueError(plan)

        container_name = self.get_container_name(container_name=container_name, model_id=model_id, framework=framework)
        if not container_name:
            raise ValueError(container_name)
        if container_name == 'vllm/vllm-openai:latest' and not ((hugging_face_id == None) ^ (model_id == None)):
            raise ValueError("Hugging Face ID is required.")

        private_image_details = {}
        if container_type == PRIVATE:
            private_image_details = {"registry_namespace_id": registry_namespace_id}
            container_name = REGISTRY+"/"+container_name

        args = convert_to_base64(str(args))
        commands = convert_to_base64(str(commands))
        payload = {
                "name": endpoint_name,
                "world_size": world_size,
                "custom_endpoint_details": {
                    "container": {
                        "container_name": container_name,
                        "container_type": container_type,
                        "private_image_details": private_image_details,
                        "advance_config": {
                            "image_pull_policy": "Always",
                            "is_readiness_probe_enabled": "false",
                            "is_liveness_probe_enabled": "false",
                        }
                    },
                    "resource_details": {
                        "disk_size": disc_size,
                        "mount_path": mount_path,
                        "env_variables": env_variables
                    },
                },
                "model_id": model_id,
                "sku_id": sku_id,
                "replica": replicas,
                "path": model_path,
                "framework": framework,
                "is_auto_scale_enabled": is_auto_scale_enabled,
                "detailed_info": {
                    "commands": commands,
                    "args": args,
                    "hugging_face_id": hugging_face_id,
                    "tokenizer": ""
                },
                "model_load_integration_id": hf_token_ingration_id,
                "dataset_id": dataset_id,
                "dataset_path": dataset_path
            }

        if is_auto_scale_enabled:
            rules = [
                {
                    "metric": metric,
                    "condition_type": "limit",
                    "value": value,
                    "watch_period": "60"
                }
            ]

            auto_scale_policy = {
                "min_replicas": min_replicas,
                "max_replicas": max_replicas,
                "rules": rules,
                "stability_period": "300"
            }
            payload["auto_scale_policy"] = auto_scale_policy

        if framework == 'custom':
            payload["custom_endpoint_details"]["container"]["advance_config"].update({
                "readiness_probe": {
                                        "protocol": readiness_probe_protocol,
                                        "initial_delay_seconds": initial_delay_seconds,
                                        "success_threshold": success_threshold,
                                        "failure_threshold": failure_threshold,
                                        "port": 8080,
                                        "period_seconds": period_seconds,
                                        "timeout_seconds": timeout_seconds,
                                        "path": probe_url,
                                        "grpc_service": "",
                                        "commands": readiness_commands
                                    },
                "liveness_probe": {
                                        "protocol": liveness_probe_protocol,
                                        "initial_delay_seconds": initial_delay_seconds,
                                        "success_threshold": success_threshold,
                                        "failure_threshold": failure_threshold,
                                        "port": 8080,
                                        "period_seconds": period_seconds,
                                        "timeout_seconds": timeout_seconds,
                                        "path": probe_url,
                                        "grpc_service": "",
                                        "commands": liveness_commands
                                    }
            })

        payload = json.dumps(payload)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        print(f"To check the Inference Status and logs, PLease visit "
              f"https://gpu-notebooks.e2enetworks.com/projects/{client.Default.project()}/model-endpoints")
        return prepare_object(response)

    def get(self, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def logs(self, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/logs/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def start(self, endpoint_id):
        payload = json.dumps({
            "action": "start"
        })
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def stop(self, endpoint_id):
        payload = json.dumps({
            "action": "stop"
        })
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def restart(self, endpoint_id):
        payload = json.dumps({
            "action": "restart"
        })
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list_replicas(self, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/replicas/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def manage_replica_count(self, replica_count, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        if replica_count < 1 or replica_count > 5:
            raise ValueError(f"Invalid replica count. Expected between 1 and 5, got {replica_count}")
        payload = json.dumps({
                              "action": "disable_auto_scale",
                              "replicas": replica_count,
                              })
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def restart_replica(self, replica_name, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        if type(replica_name)!=str:
            raise ValueError(f"Invalid replica name. Expected a string, got {replica_name}")

        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/replicas/{replica_name}/"
        req = requests.Request('DELETE', url=url, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get_integration(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/integrations/?integration_type=hugging_face&"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("EndPoint Class Help")
        print("\t\t=================")
        print("\t\tThis class provides functionalities to interact with EndPoint.")
        print("\t\tAvailable methods:")
        print("\t\t1. __init__(team, project): Initializes an EndPoints instance with the specified team and "
              "project IDs.")
        print("\t\t2. list_plans(): List the plans details")
        print("\t\t3. list_frameworks(): List the all available framework")
        print("\t\t4. get_container_name(): Get the name of the container associated with a framework")
        print("\t\t5. get_container_type(): Get the container type")
        print("\t\t6. create_triton(endpoint_name, plan, model_id, model_path='', replicas=1)")
        print("\t\t7. create_tensorrt(endpoint_name, plan, model_id, model_path='', replicas=1)")
        print("\t\t8. create_pytorch(endpoint_name, plan, model_id, model_path='', replicas=1)")
        print("\t\t9. create(endpoint_name, framework, plan, container_name, container_type, model_id, replicas=1, "
              "disc_size=10, model_path="", env_variables=[], mount_path="", registry_endpoint="", "
              "auth_type='pass', username="", password="", docker_config=""): "
              "Creates an endpoint with the provided details.")
        print("\t\t10. registry_namespace_list(): Get the list of all registry on e2enetworks")
        print("\t\t11. registry_detail(registry_namespace_id): Get details of registry")
        print("\t\t12. get(endpoint_id): Retrieves information about a specific endpoint using its ID.")
        print("\t\t13. logs(endpoint_id): Retrieves logs of a specific endpoint using its ID.")
        print("\t\t14. stop(endpoint_id): Stops a specific endpoint using its ID.")
        print("\t\t15. start(endpoint_id): Starts a specific endpoint using its ID.")
        print("\t\t16. list(): Lists all endpoints associated with the team and project.")
        print("\t\t17. restart(endpoint_id): Restart a inference")
        print("\t\t18. delete(endpoint_id): Deletes an endpoint with the given ID.")
        print("\t\t19. list_replicas(endpoint_id): List all endpoints associated replicas")
        print("\t\t20. manage_replica_count(endpoint_id): Manage replica count either increase or decrease")
        print("\t\t21. restart_replica(replica_name,endpoint_id): Restart a specific replica")
        print("\t\t22. get_integration(): Get all integration details")
        print("\t\t23. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tendpoints = EndPoints(123, 456)")
        print("\t\tendpoints.create("
              "\n\t\t\t\tendpoint_name(required):String => 'Name of Endpoint'",
              "\n\t\t\t\tframework(required):String => '['triton', 'pytorch', 'llma', 'stable_diffusion', 'mpt,"
              "\n\t\t\t\t\t'codellama', 'custom']'",
              "\n\t\t\t\tplan(required):String=> Plans Can be listed using tir.Plans Apis",
              "\n\t\t\t\tcontainer_type(optional):String=> Default value is public and "
              "\n\t\t\t\t\tallowed values are [public, private]",
              "\n\t\t\t\tmodel_id:Integer=> Required in case of Framework type=[triton, pytorch] and "
              "\n\t\t\t\t\tif model is stored in EOS",
              "\n\t\t\t\tcontainer_name(optional):String=> Docker Container Image Name required in case of Custom "
              "\n\t\t\t\tContainer Only",
              "\n\t\t\t\treplicas(optional):Integer=> Default value id 1",
              "\n\t\t\t\tdisc_size(optional):Integer=> Default value id 10Gb",
              "\n\t\t\t\tmodel_path(optional):String=> Path of EOS bucket where the model is stored",
              "\n\t\t\t\tenv_variables(optional):List=> Env variables can be passed as "
              "\n\t\t\t\t\t[{ 'key': '', 'value': '/mnt/models'}]"
              "\n\t\t\t\tmount_path(optional):String=> Default value is '/mnt/models'"
              "\n\t\t\t\tregistry_endpoint(optional):String=> Required in Case of container_type=private"
              "\n\t\t\t\tauth_type(optional):String=> Required in case of container_type=private, "
              "\n\t\t\t\t\tAllowed Values are ['pass', 'docker'] "
              "\n\t\t\t\t\tDefault Value is pass'"
              "\n\t\t\t\tusername(optional):String=> Required in case of container_type=private and auth_type=pass"
              "\n\t\t\t\tusername(optional):String=> Required in case of container_type=private and auth_type=pass"
              "\n\t\t\t\tdocker_config(optional):String=> Required in case of container_type=private and "
              "auth_type=docker")
        print("\t\tendpoints.get(789)")
        print("\t\tendpoints.logs(789)")
        print("\t\tendpoints.stop(789)")
        print("\t\tendpoints.start(789)")
        print("\t\tendpoints.list()")
        print("\t\tendpoints.delete(789)")




from e2enetworks.constants import WHISPER_LARGE_V3, LLAMA_2_13B_CHAT, STABLE_DIFFUSION_2_1, MIXTRAL_8X7B_INSTRUCT, \
    CODELLAMA_13B_INSTRUCT, E5_MISTRAL_7B_INSTRUCT, LLAMA_3_8B_INSTRUCT

ARGUMENT_IS_MANDATORY = "IS MANDATORY"
    
MODEL_NAME_TO_URL_PATH_MAPPING = {
    LLAMA_2_13B_CHAT : "project/{namespace}/v1/llama-2-13b-chat/infer",
    STABLE_DIFFUSION_2_1 : "project/{namespace}/v1/stable-diffusion-2-1/infer",
    MIXTRAL_8X7B_INSTRUCT: "project/{namespace}/v1/mixtral-8x7b-instruct/infer",
    CODELLAMA_13B_INSTRUCT : "project/{namespace}/v1/codellama-13b-instruct/infer",
    WHISPER_LARGE_V3 : "project/{namespace}/v1/whisper-large-v3/infer",
    E5_MISTRAL_7B_INSTRUCT: "project/{namespace}/v1/e5-mistral-7b-instruct/infer",
    LLAMA_3_8B_INSTRUCT: "project/{namespace}/v1/llama-3-8b-instruct/infer",
}
MODEL_API_DEFAULT_DATA = {"inputs": []}

MODELS_API_DATA_FORMATS = {
    LLAMA_2_13B_CHAT: {
        "prompt": {
            "name": "prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "system_prompt": {
            "name": "system_prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "max_new_tokens": {
            "name": "max_new_tokens",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_k": {
            "name": "top_k",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_p": {
            "name": "top_p",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "temperature": {
            "name": "temperature",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "num_return_sequences": {
            "name": "num_return_sequences",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "repetition_penalty": {
            "name": "repetition_penalty",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
    },
    LLAMA_3_8B_INSTRUCT: {
        "prompt": {
            "name": "prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "system_prompt": {
            "name": "system_prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "max_new_tokens": {
            "name": "max_new_tokens",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_k": {
            "name": "top_k",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_p": {
            "name": "top_p",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "temperature": {
            "name": "temperature",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "num_return_sequences": {
            "name": "num_return_sequences",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "repetition_penalty": {
            "name": "repetition_penalty",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
    },
    MIXTRAL_8X7B_INSTRUCT: {
        "prompt": {
            "name": "prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "system_prompt": {
            "name": "system_prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "max_new_tokens": {
            "name": "max_new_tokens",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_k": {
            "name": "top_k",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_p": {
            "name": "top_p",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "temperature": {
            "name": "temperature",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "num_return_sequences": {
            "name": "num_return_sequences",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "repetition_penalty": {
            "name": "repetition_penalty",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
    },
    CODELLAMA_13B_INSTRUCT : {
        "prompt": {
            "name": "prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "system_prompt": {
            "name": "system_prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "max_new_tokens": {
            "name": "max_new_tokens",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_k": {
            "name": "top_k",
            "shape": [1],
            "datatype": "INT32",
            "data": [],
        },
        "top_p": {
            "name": "top_p",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "temperature": {
            "name": "temperature",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
        "action": {
            "name": "action",
            "shape": [1],
            "datatype": "BYTES",
            "data": [],
        },
        "repetition_penalty": {
            "name": "repetition_penalty",
            "shape": [1],
            "datatype": "FP32",
            "data": [],
        },
    },
    STABLE_DIFFUSION_2_1: {
        "prompt": {
            "name": "prompt",
            "shape": [1, 1],
            "datatype": "BYTES",
            "data": [],
        },
        "negative_prompt": {
            "name": "negative_prompt",
            "shape": [1, 1],
            "datatype": "BYTES",
            "data": [],
        },
        "height": {
            "name": "height",
            "shape": [1, 1],
            "datatype": "UINT16",
            "data": [],
        },
        "width": {
            "name": "width",
            "shape": [1, 1],
            "datatype": "UINT16",
            "data": [],
        },
        "generator": {
            "name": "generator",
            "shape": [1, 1],
            "datatype": "UINT16",
            "data": [],
        },
        "num_inference_steps": {
            "name": "num_inference_steps",
            "shape": [1, 1],
            "datatype": "UINT16",
            "data": [],
        },
        "guidance_scale": {
            "name": "guidance_scale",
            "shape": [1, 1],
            "datatype": "FP32",
            "data": [],
        },
        "guidance_rescale": {
            "name": "guidance_rescale",
            "shape": [1, 1],
            "datatype": "FP32",
            "data": [],
        },
    },
    WHISPER_LARGE_V3 : {
        "input": {
            "name": "input",
            "shape": [1],
            "datatype": "BYTES",
            "data": []
        },
        "language": {
            "name": "language",
            "shape": [1],
            "datatype": "BYTES",
            "data": []
        },
        "task" : {
            "name": "task",
            "shape": [1],
            "datatype": "BYTES",
            "data": []
        },
        "return_timestamps" : {
            "name": "return_timestamps",
            "shape": [1],
            "datatype": "BYTES",
            "data": []
        },
        "max_new_tokens": {
            "name": "max_new_tokens",
            "shape": [1],
            "datatype": "INT32",
            "data": []
        }
    },
    E5_MISTRAL_7B_INSTRUCT : {
        "prompt": {
            "name": "prompt",
            "shape": [1],
            "datatype": "BYTES",
            "data": []
        }
    }
}

ALLOWED_TIMESTAMPS_VALUE_WHISPER = ['none', 'sentence','word']
REQUIRED_PARAMETERS_WHISPER = [ 'input', 'task',  'language']
LANGUAGES_SUPPORTED_BY_WHISPER = [
    "english",
    "hindi",
    "bengali",
    "telugu",
    "marathi",
    "tamil",
    "urdu",
    "gujarati",
    "kannada",
    "punjabi",
    "malayalam",
    "assamese",
    "sanskrit",
    "chinese",
    "spanish",
    "russian",
    "korean",
    "french",
    "japanese",
    "portuguese",
    "turkish",
    "german",
    "arabic",
    "italian",
    "indonesian",
    "vietnamese",
    "dutch"
]



import json
from typing import Optional
import time

import requests

from e2enetworks import constants
from e2enetworks.cloud.tir.helpers import plan_name_to_sku_item_price_id, plan_name_to_sku_id
from e2enetworks.cloud.tir.skus import Plans, client
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.constants import (BASE_GPU_URL, INSTANCE_TYPE, NOTEBOOK,CUSTOM, SSH_ENABLE, SSH_DISABLE, SSH_UPDATE,
                                   PAID_USAGE, headers, AUTO_RENEW_STATUS, AUTO_TERMINATE_STATUS, CONVERT_TO_HOURLY_BILLING)


class Notebooks:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def create(self, name, plan_name, image_type, image_version_id=None,
               dataset_id_list=[], disk_size_in_gb=30, is_jupyterlab_enabled=True, public_key=[],
               notebook_type="new", notebook_url="", registry_namespace_id=None, 
               e2e_registry_image_url=None, sku_item_price_id=None, commited_policy=None, 
               next_sku_item_price_id=None,):

        skus, skus_table = Plans().get_plans_list(NOTEBOOK, image_version_id)
        hourly_sku_item_price_id = plan_name_to_sku_item_price_id(skus, plan_name)
        sku_item_price_id = sku_item_price_id if sku_item_price_id else hourly_sku_item_price_id
        COMMITED_POLICY = [AUTO_RENEW_STATUS, AUTO_TERMINATE_STATUS, CONVERT_TO_HOURLY_BILLING]

        if sku_item_price_id != hourly_sku_item_price_id and not commited_policy:
            raise ValueError("commited_policy is required")
        if commited_policy and commited_policy not in COMMITED_POLICY:
            raise ValueError(f"commited_policy is should be in %s" % COMMITED_POLICY)
        
        if (commited_policy == AUTO_RENEW_STATUS or commited_policy == CONVERT_TO_HOURLY_BILLING) \
        and not next_sku_item_price_id :
            raise ValueError("next_sku_item_price_id is required")
        if commited_policy == AUTO_TERMINATE_STATUS :
            next_sku_item_price_id = None
        elif commited_policy == CONVERT_TO_HOURLY_BILLING :
            next_sku_item_price_id = hourly_sku_item_price_id
        

        if image_type!=CUSTOM and not image_version_id :
            raise ValueError("image_version_id is required")
        
        if image_type==CUSTOM and (not registry_namespace_id or not e2e_registry_image_url):
            raise ValueError("registry_namespace_id and e2e_registry_image_url are required")

        payload = json.dumps({
            "name": name,
            "dataset_id_list": dataset_id_list,
            "image_type": image_type,
            "is_jupyterlab_enabled": is_jupyterlab_enabled,
            "public_key": public_key,
            "sku_item_price_id": sku_item_price_id,
            "auto_shutdown_timeout": None,
            "instance_type": PAID_USAGE,
            "disk_size_in_gb": disk_size_in_gb,
            "notebook_type": notebook_type,
            "notebook_url": notebook_url,
        })
        new_payload = json.loads(payload)
        if image_type == CUSTOM :
            new_payload["registry_namespace_id"] = registry_namespace_id
            new_payload["e2e_registry_image_url"] = e2e_registry_image_url
        else :
            new_payload["image_version_id"] = image_version_id
        
        if commited_policy :
            new_payload["committed_instance_policy"] = commited_policy
            new_payload["next_sku_item_price_id"] = next_sku_item_price_id
        
        payload = json.dumps(new_payload)

        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        return prepare_object(response)

    def get(self, notebook_id):

        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        response = response.text
        response = json.loads(response)
        if response.get('code')==200:
            return response.get('message')
        raise ValueError("notebook not available with given id")
        return False

    def stop(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)
        
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/actions/?action=stop&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        
        if response.status_code==200:
            print("Stopping notebook. it may take a few seconds ...")
            time.sleep(10)
            print("Stopped notebook")
        return prepare_object(response)

    def start(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/actions/?action=start&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        if response.status_code==200:
            print("Starting notebook. it may take a few seconds ...")
        return prepare_object(response)

    def configure_ssh(self, notebook_id: int, action: str, ssh_keys_to_add=[], ssh_keys_to_remove=[]):
        if not isinstance(notebook_id, int):
            raise ValueError("notebook_id is not a valid integer.")
        if isinstance(ssh_keys_to_add, str):
            ssh_keys_to_add = [ssh_keys_to_add]
        if isinstance(ssh_keys_to_remove, str):
            ssh_keys_to_remove = [ssh_keys_to_remove]

        if action not in [SSH_ENABLE, SSH_DISABLE, SSH_UPDATE]:
            raise ValueError("Invalid action... available actions: ['enable', 'disable', 'update']")
        if action == SSH_UPDATE and not ssh_keys_to_add and not ssh_keys_to_remove:
            raise ValueError("'update' action requires either 'ssh_keys_to_add' or 'ssh_keys_to_remove'")
        if action == SSH_ENABLE and not ssh_keys_to_add:
            raise ValueError("'enable' action requires 'ssh_keys_to_add'")

        match action:
            case constants.SSH_DISABLE:
                disable_url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/{notebook_id}/ssh-keys/"
                req = requests.Request('DELETE', disable_url)
            case constants.SSH_ENABLE:
                enable_url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/{notebook_id}/ssh-keys/"
                req = requests.Request('PUT', enable_url, json={"ssh_keys_to_add": ssh_keys_to_add})
            case constants.SSH_UPDATE:
                update_url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/{notebook_id}/ssh-keys/"
                req = requests.Request('PUT', update_url, json={"ssh_keys_to_add": ssh_keys_to_add, "ssh_keys_to_remove": ssh_keys_to_remove})
            case _:
                raise ValueError("Invalid action...")
        response = client.Default.make_request(req)
        if response.status_code == 200:
            print("SSH configured successfully.")
        return prepare_object(response)

    def list_attached_ssh_keys(self, notebook_id: int):
        if type(notebook_id) is not int:
            raise ValueError("notebook_id is not a valid integer")
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/{notebook_id}/ssh-keys/"
        req = requests.Request('GET', url)
        return prepare_object(client.Default.make_request(req))

    def upgrade(self, notebook_id, plan_name, sku_item_price_id, commited_policy, next_sku_item_price_id=None):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)
        
        if type(plan_name) != str:
            raise ValueError(plan_name)
        
        if type(sku_item_price_id) != int:
            raise ValueError(sku_item_price_id)
        
        if type(commited_policy) != str:
            raise ValueError(next_sku_item_price_id)
        
        skus, skus_table = Plans().get_plans_list(NOTEBOOK)
        hourly_sku_item_price_id = plan_name_to_sku_item_price_id(skus, plan_name)
        sku_id = plan_name_to_sku_id(skus, plan_name)
        sku_item_price_id = sku_item_price_id if sku_item_price_id else hourly_sku_item_price_id
        COMMITED_POLICY = [AUTO_RENEW_STATUS, AUTO_TERMINATE_STATUS, CONVERT_TO_HOURLY_BILLING]

        if sku_item_price_id != hourly_sku_item_price_id and not commited_policy:
            raise ValueError("commited_policy is required")
        if commited_policy not in COMMITED_POLICY:
            raise ValueError(f"commited_policy is should be in %s" % COMMITED_POLICY)
        
        if (commited_policy == AUTO_RENEW_STATUS or commited_policy == CONVERT_TO_HOURLY_BILLING) \
        and not next_sku_item_price_id :
            raise ValueError("next_sku_item_price_id is required")
        if commited_policy == AUTO_TERMINATE_STATUS :
            next_sku_item_price_id = None
        elif commited_policy == CONVERT_TO_HOURLY_BILLING :
            next_sku_item_price_id = hourly_sku_item_price_id
        
        payload = json.dumps(
                        {
                    "sku_id": sku_id,
                    "sku_item_price_id": sku_item_price_id,
                    "committed_instance_policy": commited_policy,
                    "next_sku_item_price_id": next_sku_item_price_id
                }
        )

        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/?apikey={client.Default.api_key()}"
        response = requests.put(url=url, headers=headers, data=payload)
        return prepare_object(response)

    def upgrade_pvc(self, notebook_id, size):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        if type(size) != int:
            raise ValueError(notebook_id)

        payload = json.dumps({
            "size": size})
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/pvc/upgrade/?apikey={client.Default.api_key()}"
        response = requests.put(url=url, headers=headers, data=payload)
        return prepare_object(response)
    
    def registry_namespace_list(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/container_registry/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)
    
    def registry_detail(self, registry_namespace_id):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/container_registry/{registry_namespace_id}/namespace-repository/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)
    
    def commited_policy(self):
        INSTANCE_UPDATION_LIST = [AUTO_RENEW_STATUS, AUTO_TERMINATE_STATUS, CONVERT_TO_HOURLY_BILLING]
        print(INSTANCE_UPDATION_LIST)
        

    @staticmethod
    def help():
        print("\t\tNotebook Class Help")
        print("\t\t=================")
        help_text = f"""
                Notebooks class provides methods to interact with notebooks in a project.

                Available methods:
                1. create(
                    name(required):String, 
                    plan(required):String=> Plans Can be listed using tir. Plans Apis",
                    image_id(required), 
                    instance_type={INSTANCE_TYPE}, 
                    disk_size_in_gb=30,
                    notebook_type="new",
                    notebook_url="",
                    is_jupyterlab_enabled: bool,
                    public_key: list (required if JupyterLab is disabled))
                2. get(notebook_id): Get information about a notebook.
                3. list(): List all notebooks in the project.
                4. delete(notebook_id): Delete a notebook.
                5. stop(notebook_id): Stop a running notebook.
                6. start(notebook_id): Start a stopped notebook.
                7. upgrade(notebook_id, size): Upgrade the size of a notebook's PVC.
                8. list_attached_ssh_keys(notebook_id): List all attached ssh keys.
                9. configure_ssh(notebook_id, action('enable', 'disable', 'update'), ssh_keys_to_add(optional), ssh_keys_to_remove(optional)): Enable, disable or update ssh keys by adding or removing keys.

                Usage:
                notebooks = Notebooks(team, project)
                notebooks.create("test-notebook", "CPU-C3-4-8GB-0", 9, 
                    instance_type="paid_usage", 
                    disk_size_in_gb=30,
                    notebook_type="new",
                    notebook_url="")
                notebooks.get(notebook_id)
                notebooks.list()
                notebooks.delete(notebook_id)
                notebooks.stop(notebook_id)
                notebooks.start(notebook_id)
                notebooks.upgrade_pvc(notebook_id, size)
                notebooks.list_attached_ssh_keys(notebook_id)
                notebooks.configure_ssh(notebook_id, action, ssh_keys_to_add(optional), ssh_keys_to_remove(optional))
                """
        print(help_text)



import json
import os
from typing import Optional
from tqdm import tqdm
from tqdm.utils import CallbackIOWrapper
import requests

from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.constants import MODEL_NAME_TO_URL_PATH_MAPPING, LANGUAGES_SUPPORTED_BY_WHISPER,REQUIRED_PARAMETERS_WHISPER,ALLOWED_TIMESTAMPS_VALUE_WHISPER
from e2enetworks.cloud.tir.helpers import (get_formated_data_for_model,
                                           get_model_url, get_random_string)
from e2enetworks.cloud.tir.minio_service import MinioService
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.constants import (BASE_GPU_URL, BUCKET_TYPES, MANAGED_STORAGE,
                                   MODEL_TYPES, headers,WHISPER_LARGE_V3,STABLE_DIFFUSION_2_1,WHISPER_DATA_LIMIT_BYTES)


class ModelAPIClient:
    def __init__(self, team: Optional[str] = "", project: Optional[str] = ""):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def infer(self, model_name, data):
        
        if model_name == WHISPER_LARGE_V3 :
            return self.__whisper_infer(data)
        
        if model_name == STABLE_DIFFUSION_2_1 and (not data.get('generator') or data.get('generator')=='null') :
            raise ValueError('generator should not be null')
        
        namespace = "p-" + str(client.Default.project())
        try:
            url_status, url = get_model_url(model_name, namespace)
            if not url_status:
                raise Exception(f"Invalid input Error {url} model not present")
            data_status, payload = get_formated_data_for_model(model_name, data)
            if not data_status:
                raise Exception(f"Invalid data key Error {str(payload)}")

        except Exception as e:
            raise Exception(f"Input Error {e}")
        headers["Authorization"] = f"Bearer {client.Default.access_token()}"
        response = requests.post(url=url, headers=headers, data=json.dumps(payload))
        response = prepare_object(response)
        return response
    
    def __whisper_infer(self, data : dict):
        namespace = "p-" + str(client.Default.project())
        try:
            is_valid_input, error = self.__validate_whisper_data(data)

            if not is_valid_input :
                raise Exception(error)
            
            url_status, url = get_model_url(WHISPER_LARGE_V3, namespace)
            if not url_status:
                raise Exception({
                    "error_type" : "Input Error",
                    "message" : f"Invalid input Error {url} model not present"
                })
            data_status, api_payload = get_formated_data_for_model(WHISPER_LARGE_V3, data)
            if not data_status:
                raise Exception({
                    "error_type" : "Input Error",
                    "message" : f"Invalid data key Error {str(data)}"
                })
            
            upload_response = self.__upload_data_genai(data["input"])
            if upload_response and upload_response.get("code") == 200:
                uploaded_file_path = upload_response["data"]["uploaded_file_url"]
                api_payload["inputs"][0]['data'] = [uploaded_file_path]
            else :
                return {
                     "code": upload_response.get("code"),
                     "message" : upload_response.get("message")
                }
                
        except Exception as e:

            return {
                "status": "error",
                "message" : "Error Occured",
                "data" : {},
                "error" : e,
            }
        
        
        headers["Authorization"] = f"Bearer {client.Default.access_token()}"
        print("Inference Requested")
        infer_response = requests.post(url=url, headers=headers, data=json.dumps(api_payload))
        infer_response = prepare_object(infer_response)
        return infer_response
        
    def __upload_data_genai(self,file_path):

        if os.path.exists(file_path) and os.path.isfile(file_path) :
            file_size = os.path.getsize(file_path)
            file_name = os.path.basename(file_path)
            if file_size <= WHISPER_DATA_LIMIT_BYTES :
                headers['Authorization'] = f'Bearer {client.Default.access_token()}'
                url_to_get_upload_url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/datasets/genai/objects/urls/?"\
                                        f"apikey={client.Default.api_key()}&file_name={file_name}&action=upload" 
                response_get_upload_url = requests.get(url_to_get_upload_url,headers=headers)
                response_get_upload_url = response_get_upload_url.json()
                
                if response_get_upload_url.get("code") != 200:
                    return response_get_upload_url
                
                uploaded_file_url = response_get_upload_url["data"].get("model_input_url")
                file_upload_url = response_get_upload_url["data"].get("upload_url")
                
                put_instance = requests.Session()
                put_instance.headers.update({'content-type': 'multipart/form-data'})
                put_instance.timeout = 600 

                with open(file_path, "rb") as fd:
                    with tqdm(desc=f"Uploading Audio File '{file_name}' ", total=file_size, unit="B", unit_scale=True, unit_divisor=1024) as t:
                        reader_wrapper = CallbackIOWrapper(t.update, fd, "read")
                        file_upload_response = requests.put(file_upload_url, data=reader_wrapper)
                        file_upload_response.raise_for_status()
                        
                file_upload_status_code = file_upload_response.status_code
                if file_upload_status_code == 200:

                    return {
                        "code": file_upload_status_code,
                        "message": "Audio File Uploaded Successfully",
                        "data": {
                            "uploaded_file_url" : uploaded_file_url
                        }
                    }
                else : 
                    raise Exception({
                        "error_type" : "Audio File Upload Error",
                        "message" : f"Internal Error. Failed to upload file."
                    })
                
            else :
                 raise Exception({
                    "error_type" : "Audio File Error",
                    "message" : f"The audio file size exceeds the maximum limit of 50 Megabytes. The current file size is {round(file_size/(pow(1024,2)), 2)} Megabytes."
                })
        else :
            raise Exception({
                "error_type" : "Audio File Error",
                "message" : "File not found or file does not exist on the 'input' "
            })
    
    def __validate_whisper_data(self, data : dict):

        for parameter in REQUIRED_PARAMETERS_WHISPER :
            if parameter not in data.keys():
                return False, {
                    "error_type" : "Input Error",
                    "message" : f"The necessary parameter '{parameter}' is missing."
                    }
        
        if data["language"].lower() not in LANGUAGES_SUPPORTED_BY_WHISPER :
            return False, {
                "error_type" : "Input Error",
                "message" :  f"Invalid 'language', Following are the allowed values {LANGUAGES_SUPPORTED_BY_WHISPER}"
            }
        
        if  data.get("max_new_tokens")  and (data["max_new_tokens"] < 0 or data["max_new_tokens"] > 445 ) :
            return False, {
                "error_type" : "Input Error",
                "message" :  f"'max_new_tokens' should be in the range [1,445]."
            }
        
        if data.get("return_timestamps") and data['return_timestamps'] not in ALLOWED_TIMESTAMPS_VALUE_WHISPER :
            return False,{
                "error_type" : "Input Error",
                "message" :  f"Invalid 'return_timestamps', Following are the allowed values {ALLOWED_TIMESTAMPS_VALUE_WHISPER}"
            }
            
            
        return True, {}
                
   
    def list(self):
        response = MODEL_NAME_TO_URL_PATH_MAPPING.keys()
        return list(response)

    @staticmethod
    def help():
        print("ModelAPIClient Class Help")
        print("\t\t=================")
        print("\t\tThis class provides functionalities to infer with models.")
        print("\t\tAvailable methods:")
        print(
            "\t\t1. __init__(team, project): Initializes a Models instance with the specified team and project "
            "IDs."
        )
        print("\t\t2. list(): List all available models" "details.")
        print(
            "\t\t3. infer(model_name, data): Infer model with the provided "
            "details."
        )
        print("\t\t4. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tmodelclient = ModelAPIClient(123, 456)")
        print(f"\t\tmodelclient.list()")
        print(f"\t\tmodelclient.infer(model_name, data)")
